
local Settings = {
	Version = "4.00",
	Debug = false,
	Enabled = true,
	Player = UnitName("player");
	TextColor = 
		{
			R = 0.7,
			G = 0.7,
			B = 1.0
		}
};
 
local LoadedOptions = {};

--[[
	Event Handlers
]]--
function QuestAnnounce:OnLoad()
	QuestAnnounceFrame:RegisterEvent("ADDON_LOADED");
	
	DEFAULT_CHAT_FRAME:AddMessage("QuestAnnounce v."..QuestAnnounce.Version, Defaults.TextColor.R, Defaults.TextColor.G, Defaults.TextColor.B);

	SlashCmdList["QUESTANNOUNCE"] = function(msg) QuestAnnounce:SlashHandler(msg or nil) end;
	SLASH_QUESTANNOUNCE1 = "/qa";
	SLASH_QUESTANNOUNCE2 = "/questannounce";
end

function QuestAnnounce:OnEvent(self, event, ...)
	local message = ...;
	if (event == "ADDON_LOADED") and message == "QuestAnnounce" then
		self:SendDebugMsg("ADDON_LOADED: "..message);
	  	QuestAnnounceTooltip:SetOwner(UIParent, "ANCHOR_NONE");
		
		self:RegisterEvent("UI_INFO_MESSAGE");
	
		if (not Options.Settings) then
			QuestAnnounceOptions = {};
			QuestAnnounceOptions.Settings = {};
			QuestAnnounceOptions.Settings[QuestAnnounce.Player] = {};
		elseif (QuestAnnounceOptions.Settings[QuestAnnounce.Player]) then
			QuestAnnounceOptions.Settings.Enabled = QuestAnnounceOptions.Settings[QuestAnnounce.Player].Enabled;
			QuestAnnounceOptions.Settings.TextColor.R = QuestAnnounceOptions.Settings[QuestAnnounce.Player].TextColor.R;
			QuestAnnounceOptions.Settings.TextColor.G = QuestAnnounceOptions.Settings[QuestAnnounce.Player].TextColor.G;
			QuestAnnounceOptions.Settings.TextColor.B = QuestAnnounceOptions.Settings[QuestAnnounce.Player].TextColor.B;
		else
			QuestAnnounce_SendFunctionalMsg("QuestAnnounce Defaults Settings Loaded - type '/qa' for options.");
			QuestAnnounceOptions.Settings[QuestAnnounce.Player] = {};
		end
		
		QuestAnnounce:Config();
		QuestAnnounce:SaveSettings();
	end
	
	if (event == "UI_INFO_MESSAGE" and message ~= nil) then
		self:SendDebugMsg("UI_INFO_MESSAGE: "..message);
		if (QuestAnnounceOptions.Settings.Enabled) then
			local questText = gsub(message,"(.*):%s*([-%d]+)%s*/%s*([-%d]+)%s*$","%1",1);
			
			if (questText ~= message) then
				local ii, jj, strItemName, iNumItems, iNumNeeded = string.find(message, "(.*):%s*([-%d]+)%s*/%s*([-%d]+)%s*$");
				local stillNeeded = iNumNeeded - iNumItems;
                   
				if (stillNeeded > 0) then
					self:SendPartyMsg("Progress: "..message);
					self:SendDebugMsg("Progress: "..message);
				else
					self:SendPartyMsg("Completed Objective: "..message);
					self:SendDebugMsg("Completed Objective: "..message);
				end
			end
		end
	end
end

function QuestAnnounce:SlashHandler(msg)
	if (string.lower(msg) == "debug") then
		QuestAnnounce.Debug = not QuestAnnounce.Debug;
		QuestAnnounce:SaveSettings();
		self:SendFunctionalMsg("QuestAnnounce Debug Mode: "..(QuestAnnounce.Debug and "|c0033CC33On" or "|c00CC3333Off"));
  	end

	if (string.lower(msg) == "enable") then
		QuestAnnounce.Settings.Enable = true;
		QuestAnnounce:SaveSettings();
		self:SendFunctionalMsg("QuestAnnounce Enabled: "..(QuestAnnounce.Settings.Enabled and "|c0033CC33On" or "|c00CC3333Off"));
  	end

	if (string.lower(msg) == "disable") then
		QuestAnnounce.Settings.Enable = nil;
		QuestAnnounce:SaveSettings();
		self:SendFunctionalMsg("QuestAnnounce Enabled: "..(QuestAnnounce.Settings.Enabled and "|c0033CC33On" or "|c00CC3333Off"));
  	end

	if (string.lower(msg) == "status") then
		self:SendFunctionalMsg("QuestAnnounce v."..QuestAnnounce.Version);
		self:SendFunctionalMsg("  Enabled: "..(QuestAnnounce.Settings.Enabled and "|c0033CC33On" or "|c00CC3333Off"));
		self:SendFunctionalMsg("  Debug: "..(QuestAnnounce.Debug and "|c0033CC33On" or "|c00CC3333Off"));
	end
	
	if(msg == "" or string.lower(msg) == "help") then
		self:SendFunctionalMsg("QuestAnnounce v."..QuestAnnounce.Version.." Command Line Arguments");
		self:SendFunctionalMsg("  |c00CC3333Enable|r: Enables QuestAnnounce.");
		self:SendFunctionalMsg("  |c00CC3333Disable|r: Disables QuestAnnounce");
		self:SendFunctionalMsg("  |c00CC3333Debug|r: Enables/Disables QuestAnnounce Debug Messages");
	end	
end


--[[
	Configuration Screen
]]--
function QuestAnnounce:Config()
	QuestAnnounce:SendDebugMsg("QuestAnnounce:Config");
	
	local configPanel = CreateFrame("FRAME","QuestAnnounce_MainFrame");
	configPanel.name = "Quest Announce";
	InterfaceOptions_AddCategory(configPanel);

	local messageHeader = configPanel:CreateFontString(nil, "ARTWORK","GameFontNormalLarge");
	messageHeader:SetPoint("TOPLEFT", 10, -10);
	messageHeader:SetText("Quest Announce v."..QuestAnnounce.Version);

	local enableButton = CreateFrame("CheckButton", "QuestAnnounce_EnabledButton", configPanel, "ChatConfigCheckButtonTemplate");
	enableButton:SetPoint("TOPLEFT", 10, -65)
	enableButton.tooltip = "Enable/Disable quest announcements to be sent to party chat."
	getglobal(enableButton:GetName().."Text"):SetText(" Enable/Disable Quest Announce");

	local debugButton = CreateFrame("CheckButton", "QuestAnnounce_DebugButton", configPanel, "ChatConfigCheckButtonTemplate");
	debugButton:SetPoint("TOPLEFT", 10, -95)
	debugButton.tooltip = "Enable/Disable debug announcements to be sent to the default chat frame."
	getglobal(debugButton:GetName().."Text"):SetText(" Enable/Disable Debugging");

	local colorButton = CreateFrame("Button", "QuestAnnounce_ColorButton", configPanel, "QuestAnnounceColorTemplate");
	colorButton:SetPoint("TOPLEFT", 39, -125)
	colorButton.tooltip = "Change the color of the quest announcements."

	local colorMessageHeader = configPanel:CreateFontString(nil, "ARTWORK","GameFontNormal");
	colorMessageHeader:SetPoint("TOPLEFT", 60, -128);
	colorMessageHeader:SetTextColor(QuestAnnounce.Settings.TextColor.R, QuestAnnounce.Settings.TextColor.G, QuestAnnounce.Settings.TextColor.B);
	colorMessageHeader:SetText("Quest Announce Text Color");

	configPanel.okay = 
		function (self)
			QuestAnnounce.Settings.Enabled = enabledButton:GetChecked();
			QuestAnnounce.Debug = debugButton:GetChecked();
			QuestAnnounce.Settings.TextColor = 
			{
				R = colorButton.r;
				G = colorButton.g;
				B = colorButton.b;
			};
			QuestAnnounce:SaveSettings();
		end
	
	configPanel.cancel = 
		function (self)
			QuestAnnounce.Settings.Enabled = QuestAnnounceOptions.Settings[QuestAnnounce.Player].Enabled;
			QuestAnnounce.Settings.TextColor = 
			{
				R = QuestAnnounceOptions.Setting[QuestAnnounce.Player].TextColor.R;
				G = QuestAnnounceOptions.Setting[QuestAnnounce.Player].TextColor.G;
				B = QuestAnnounceOptions.Setting[QuestAnnounce.Player].TextColor.B;
			};
			QuestAnnounce:SaveSettings();
		end
	
	configPanel.default = 
		function (self)
			QuestAnnounce:LoadSettings();
			QuestAnnounce:SaveSettings();
		end
end

function QuestAnnounce:SetColor(button, r, g, b)
	button.r = r;
	button.g = g;
	button.b = b;
	getglobal(button:GetName().."NormalTexture"):SetVertexColor(r, g, b);
end

function QuestAnnounce:OpenColorPicker(button)
	CloseMenus()
	if (not button) then
	  button = self;
	end
	ColorPickerFrame.func = function() QuestAnnounce:SetColor(button, ColorPickerFrame:GetColorRGB()) end;
	ColorPickerFrame:SetColorRGB(button.r, button.g, button.b);
	ColorPickerFrame.previousValues = {r = button.r, g = button.g, b = button.b, opacity = button.opacity};
	ColorPickerFrame.cancelFunc = function() QuestAnnounce:SetColor(button, ColorPickerFrame.previousValues.r, ColorPickerFrame.previousValues.g, ColorPickerFrame.previousValues.b) end;
	ColorPickerFrame:Show();
end

--[[
	Settings (Load/Save) Handling Functions
]]--
function QuestAnnounce:SaveSettings()
	getglobal("QuestAnnounce_EnabledButton"):SetChecked(QuestAnnounce.Settings.Enabled);
	getglobal("QuestAnnounce_DebugButton"):SetChecked(QuestAnnounce.Debug);
	
	QuestAnnounce:SetColor(getglobal("QuestAnnounce_ColorButton"), QuestAnnounce.Settings.TextColor.R, QuestAnnounce.Settings.TextColor.G, QuestAnnounce.Settings.TextColor.B);	

	QuestAnnounceOptions.Settings[QuestAnnounce.Player].Enabled = QuestAnnounce.Settings.Enabled;
	QuestAnnounceOptions.Settings[QuestAnnounce.Player].TextColor = 
		{
			R = QuestAnnounce.Settings.TextColor.R;
			G = QuestAnnounce.Settings.TextColor.G;
			B = QuestAnnounce.Settings.TextColor.B;
		};
	
	self:SendDebugMsg("QuestAnnounce:SaveSettings - Settings Saved");
end

function QuestAnnounce:LoadSettings()
	QuestAnnounce.Settings.Enabled = Defaults.Enabled;
	QuestAnnounce.Settings.TextColor = {};
	QuestAnnounce.Settings.TextColor.R = Defaults.TextColor.R;
	QuestAnnounce.Settings.TextColor.G = Defaults.TextColor.G;
	QuestAnnounce.Settings.TextColor.B = Defaults.TextColor.B;
	
	self:SendDebugMsg("QuestAnnounce:LoadOptions - Default Settings Loaded");
end

-- Load the default settings.
QuestAnnounce:LoadSettings();

--[[
	Message Handling Functions
]]--
function QuestAnnounce:SendPartyMsg(msg)	
	if (msg and QuestAnnounceOptions.Settings.Enabled and GetNumPartyMembers() > 0) then
		SendChatMessage(msg, "PARTY");
	end
	self:SendDebugMsg("QuestAnnounce:SendPartyMsg - "..msg);
end

function QuestAnnounce:SendDebugMsg(msg)
	if (msg and QuestAnnounce.Debug) then
		DEFAULT_CHAT_FRAME:AddMessage("QA v."..QuestAnnounce.Version.." DEBUG :: "..msg, 0.75, 1.0, 0.25);
	end
end

function QuestAnnounce_SendErrorMsg(msg)
	if (msg) then
		DEFAULT_CHAT_FRAME:AddMessage("QA v."..QuestAnnounce.Version.." ERR :: "..msg, 0.8, 0.3, 0.3);
	end
end

function QuestAnnounce:SendFunctionalMsg(msg)
	if (msg) then
		DEFAULT_CHAT_FRAME:AddMessage(msg, Defaults.TextColor.R, Defaults.TextColor.G, Defaults.TextColor.B);
	end
end